/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef WINDTRAPINTERFACE_H
#define WINDTRAPINTERFACE_H

#include "DefaultStructureInterface.h"

#include <FileClasses/FontManager.h>

#include <PlayerClass.h>

#include <GUI/Label.h>

#include <misc/string_util.h>

class WindTrapInterface : public DefaultStructureInterface {
public:
	static WindTrapInterface* Create(int ObjectID) {
		WindTrapInterface* tmp = new WindTrapInterface(ObjectID);
		tmp->pAllocated = true;
		return tmp;
	}

protected:
	WindTrapInterface(int ObjectID) : DefaultStructureInterface(ObjectID) {
		Main_HBox.AddWidget(&TextVBox);

		TextVBox.AddWidget(&Label_RequiredEnergy);
		TextVBox.AddWidget(&Label_ProducedEnergy);
		TextVBox.AddWidget(VSpacer::Create(Main_HBox.GetSize().y));
	}

	/**
		This method updates the object interface.
		If the object doesn't exists anymore then update returns false.
		\return true = everything ok, false = the object container should be removed
	*/
	virtual bool update() {
		ObjectClass* pObject = currentGame->getObjectManager().getObject(ObjectID);
		if(pObject == NULL) {
			return false;
		}

		PlayerClass* pOwner = pObject->getOwner();

        Label_RequiredEnergy.SetText(" Required: " + stringify(pOwner->getPowerRequirement()));
        Label_ProducedEnergy.SetText(" Produced: " + stringify(pOwner->getProducedPower()));

		return DefaultStructureInterface::update();
	}

private:
    VBox    TextVBox;

	Label   Label_RequiredEnergy;
	Label   Label_ProducedEnergy;
};

#endif // WINDTRAPINTERFACE_H
