/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <FileClasses/TextManager.h>

#include <globals.h>

#include <FileClasses/FileManager.h>


TextManager::TextManager() {
    // load all breifing texts
	SDL_RWops* text_lng[3];
	text_lng[0] = pFileManager->OpenFile("TEXTA." + settings.General.LanguageExt);
	text_lng[1] = pFileManager->OpenFile("TEXTO." + settings.General.LanguageExt);
	text_lng[2] = pFileManager->OpenFile("TEXTH." + settings.General.LanguageExt);

	for(int i=0;i<3;i++) {
		if(text_lng[i] == NULL) {
			fprintf(stderr,"TextManager::TextManager: Can not open language file\n");
			exit(EXIT_FAILURE);
		}
		BriefingStrings[i] = std::shared_ptr<BriefingText>(new BriefingText(text_lng[i]));
		SDL_RWclose(text_lng[i]);
	}


	// load dune text
    SDL_RWops* dune_lng = pFileManager->OpenFile("DUNE." + settings.General.LanguageExt);
	pDuneText = std::shared_ptr<IndexedTextFile>(new IndexedTextFile(dune_lng));
	SDL_RWclose(dune_lng);
}

TextManager::~TextManager() {
}

std::string	TextManager::GetBriefingText(unsigned int mission, unsigned int texttype, int house) {
	switch(house) {
		case HOUSE_ATREIDES:
		case HOUSE_FREMEN:
			return BriefingStrings[0]->getMissionBriefingString(mission,texttype);
			break;
		case HOUSE_ORDOS:
		case HOUSE_MERCENARY:
			return BriefingStrings[1]->getMissionBriefingString(mission,texttype);
			break;
		case HOUSE_HARKONNEN:
		case HOUSE_SARDAUKAR:
		default:
			return BriefingStrings[2]->getMissionBriefingString(mission,texttype);
			break;
		}
}
