/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <Menu/OptionsMenu.h>

#include <globals.h>

#include <main.h>

#include <FileClasses/GFXManager.h>
#include <FileClasses/INIFile.h>

#include <string.h>
#include <algorithm>

OptionsMenu::OptionsMenu() : MenuClass()
{
	// set up window
	SDL_Surface *surf;
	surf = pGFXManager->getUIGraphic(UI_MenuBackground);

	SetBackground(surf,false);
	Resize(surf->w,surf->h);

	SetWindowWidget(&WindowWidget);

	newConcreteRequired = settings.General.ConcreteRequired;
	newFogOfWar = settings.General.FogOfWar;
	newDoubleBuffered = settings.Video.DoubleBuffered;
	newFullscreen = settings.Video.Fullscreen;
	newWidth = settings.Video.Width;
	newHeight = settings.Video.Height;
	newIntro = settings.General.PlayIntro;
	newLanguage = settings.General.Language;

	WindowWidget.AddWidget(&MainVBox, Point(70,70), Point(GetSize().x - 140,GetSize().y - 140));

	TextBox_Name.SetText(settings.General.PlayerName);
	MainVBox.AddWidget(&TextBox_Name);

	MainVBox.AddWidget(VSpacer::Create(5));

	Button_Concrete.SetOnClickMethod(MethodPointer(this,(WidgetCallbackWithNoParameter) (&OptionsMenu::OnOptionsConcrete)));
	MainVBox.AddWidget(&Button_Concrete);

	MainVBox.AddWidget(VSpacer::Create(5));

    Button_FogOfWar.SetOnClickMethod(MethodPointer(this,(WidgetCallbackWithNoParameter) (&OptionsMenu::OnOptionsFogOfWar)));
	MainVBox.AddWidget(&Button_FogOfWar);

	MainVBox.AddWidget(VSpacer::Create(5));

	Button_Intro.SetOnClickMethod(MethodPointer(this,(WidgetCallbackWithNoParameter) (&OptionsMenu::OnOptionsIntro)));
	MainVBox.AddWidget(&Button_Intro);

	MainVBox.AddWidget(VSpacer::Create(5));

	Button_Language.SetOnClickMethod(MethodPointer(this,(WidgetCallbackWithNoParameter) (&OptionsMenu::OnOptionsLanguage)));
	MainVBox.AddWidget(&Button_Language);

	MainVBox.AddWidget(VSpacer::Create(5));

	Button_Res.SetOnClickMethod(MethodPointer(this,(WidgetCallbackWithNoParameter) (&OptionsMenu::OnOptionsRes)));
	MainVBox.AddWidget(&Button_Res);

	MainVBox.AddWidget(VSpacer::Create(5));

	Button_FullScreen.SetOnClickMethod(MethodPointer(this,(WidgetCallbackWithNoParameter) (&OptionsMenu::OnOptionsFullscreen)));
	MainVBox.AddWidget(&Button_FullScreen);

	MainVBox.AddWidget(VSpacer::Create(5));

	Button_DoubleBuffered.SetOnClickMethod(MethodPointer(this,(WidgetCallbackWithNoParameter) (&OptionsMenu::OnOptionsDoubleBuffered)));
	MainVBox.AddWidget(&Button_DoubleBuffered);

	MainVBox.AddWidget(VSpacer::Create(5));

	OkCancel_HBox.AddWidget(Spacer::Create());

	Button_Cancel.SetText("Cancel");
	Button_Cancel.SetOnClickMethod(MethodPointer(this,(WidgetCallbackWithNoParameter) (&OptionsMenu::OnOptionsCancel)));
	OkCancel_HBox.AddWidget(&Button_Cancel);

	OkCancel_HBox.AddWidget(Spacer::Create());

	Button_Ok.SetText("Ok");
	Button_Ok.SetOnClickMethod(MethodPointer(this,(WidgetCallbackWithNoParameter) (&OptionsMenu::OnOptionsOK)));
	OkCancel_HBox.AddWidget(&Button_Ok);

	OkCancel_HBox.AddWidget(Spacer::Create());

	MainVBox.AddWidget(&OkCancel_HBox);

	determineAvailableScreenResolutions();

	UpdateButtonCaptions();
}

OptionsMenu::~OptionsMenu()
{
	;
}

void OptionsMenu::OnOptionsConcrete() {
	newConcreteRequired = !newConcreteRequired;
	UpdateButtonCaptions();
}

void OptionsMenu::OnOptionsFogOfWar() {
	newFogOfWar = !newFogOfWar;
	UpdateButtonCaptions();
}

void OptionsMenu::OnOptionsIntro() {
	newIntro = !newIntro;
	UpdateButtonCaptions();
}

void OptionsMenu::OnOptionsLanguage() {
	if(newLanguage == LNG_ENG) {
		newLanguage = LNG_FRE;
	} else if(newLanguage == LNG_FRE) {
		newLanguage = LNG_GER;
	} else {
		newLanguage = LNG_ENG;
	}
	UpdateButtonCaptions();
}

void OptionsMenu::OnOptionsRes() {
	std::list<Coord>::const_iterator iter;
	for(iter = availScreenRes.begin(); iter != availScreenRes.end(); ++iter) {
		if(iter->x == newWidth && iter->y == newHeight) {
			break;
		}
	}

	if(iter == availScreenRes.end()) {
		// the choosen resolution was hand-edited in the config file
		Coord currentRes(newWidth, newHeight);
		availScreenRes.push_back(currentRes);
		iter = availScreenRes.begin();
	} else {
		++iter;
		if(iter == availScreenRes.end()) {
			// warp around
			iter = availScreenRes.begin();
		}
	}

	newWidth = iter->x;
	newHeight = iter->y;

	UpdateButtonCaptions();
}

void OptionsMenu::OnOptionsFullscreen() {
	newFullscreen = !newFullscreen;
	UpdateButtonCaptions();
}

void OptionsMenu::OnOptionsDoubleBuffered() {
	newDoubleBuffered = !newDoubleBuffered;
	UpdateButtonCaptions();
}

void OptionsMenu::OnOptionsOK() {
	settings.General.ConcreteRequired = newConcreteRequired;
	settings.General.FogOfWar = newFogOfWar;
	settings.General.PlayIntro = newIntro;
	settings.Video.DoubleBuffered = newDoubleBuffered;
	settings.Video.Fullscreen = newFullscreen;
	settings.General.PlayerName = TextBox_Name.GetText();
	settings.Video.Width = newWidth;
	settings.Video.Height = newHeight;
	settings.General.setLanguage(newLanguage);

	SaveConfiguration2File();

	quit(1);
}

void OptionsMenu::OnOptionsCancel() {
	quit();
}

void OptionsMenu::UpdateButtonCaptions() {
	if(newConcreteRequired == true) {
		Button_Concrete.SetText("Concrete Required");
	} else {
		Button_Concrete.SetText("Concrete Not Required");
	}

    if(newFogOfWar == true) {
		Button_FogOfWar.SetText("Fog of War enabled");
	} else {
		Button_FogOfWar.SetText("Fog of War disabled");
	}

	if(newFullscreen == true) {
		Button_FullScreen.SetText("Full Screen");
	} else {
		Button_FullScreen.SetText("Windowed");
	}

	if(newDoubleBuffered == true) {
		Button_DoubleBuffered.SetText("Double Buffered");
	} else {
		Button_DoubleBuffered.SetText("Single Buffered");
	}

	if(newIntro == true) {
		Button_Intro.SetText("Play Intro");
	} else {
		Button_Intro.SetText("Don't play Intro");
	}

	if(newLanguage == LNG_ENG) {
		Button_Language.SetText("English");
	} else if(newLanguage == LNG_FRE) {
		Button_Language.SetText("French");
	} else if(newLanguage == LNG_GER) {
		Button_Language.SetText("German");
	} else {
		Button_Language.SetText("Unknown");
	}

	char temp[20];
	sprintf(temp,"%dx%d",newWidth,newHeight);
	Button_Res.SetText(temp);
}

void OptionsMenu::SaveConfiguration2File() {
	INIFile myINIFile(GetConfigFilepath());

	myINIFile.setBoolValue("General","Concrete Required",settings.General.ConcreteRequired);
    myINIFile.setBoolValue("General","Fog of War",settings.General.FogOfWar);
	myINIFile.setBoolValue("General","Play Intro",settings.General.PlayIntro);
	myINIFile.setIntValue("Video","Width",settings.Video.Width);
	myINIFile.setIntValue("Video","Height",settings.Video.Height);
	myINIFile.setBoolValue("Video","Fullscreen",settings.Video.Fullscreen);
	myINIFile.setBoolValue("Video","Double Buffered",settings.Video.DoubleBuffered);
	myINIFile.setStringValue("General","Player Name",settings.General.PlayerName);
	myINIFile.setStringValue("General","Language",settings.General.getLanguageString());

	myINIFile.SaveChangesTo(GetConfigFilepath());
}

void OptionsMenu::determineAvailableScreenResolutions() {
    availScreenRes.clear();

	SDL_Rect** pResolutions = SDL_ListModes(NULL , SDL_HWSURFACE | SDL_FULLSCREEN);

	if(pResolutions == NULL || pResolutions == (SDL_Rect**) -1) {
		// Not possible or not available
		// try some standard resolutions

		availScreenRes.push_back( Coord(640, 480) );    // VGA (4:3)
		availScreenRes.push_back( Coord(800, 480) );    // WVGA (5:3)
		availScreenRes.push_back( Coord(800, 600) );    // SVGA (4:3)
		availScreenRes.push_back( Coord(960, 540) );    // Quarter HD (16:9)
		availScreenRes.push_back( Coord(960, 720) );    // ? (4:3)
		availScreenRes.push_back( Coord(1024, 576) );   // WSVGA (16:9)
		availScreenRes.push_back( Coord(1024, 640) );   // ? (16:10)
		availScreenRes.push_back( Coord(1024, 768) );   // XGA (4:3)
		availScreenRes.push_back( Coord(1152, 864) );   // XGA+ (4:3)
		availScreenRes.push_back( Coord(1280, 720 ) );  // WXGA (16:9)
		availScreenRes.push_back( Coord(1280, 768) );   // WXGA (5:3)
		availScreenRes.push_back( Coord(1280, 800) );   // WXGA (16:10)
		availScreenRes.push_back( Coord(1280, 960) );   // SXGA- (4:3)
		availScreenRes.push_back( Coord(1280, 1024) );  // SXGA (5:4)
		availScreenRes.push_back( Coord(1366, 768) );   // HDTV 720p (~16:9)
		availScreenRes.push_back( Coord(1400, 1050) );  // SXGA+ (4:3)
		availScreenRes.push_back( Coord(1440, 900) );   // WXGA+ (16:10)
		availScreenRes.push_back( Coord(1440, 1080) );  // ? (4:3)
		availScreenRes.push_back( Coord(1600, 1200) );  // UXGA (4:3)
		availScreenRes.push_back( Coord(1680, 1050) );  // WSXGA+ (16:10)
		availScreenRes.push_back( Coord(1920, 1080) );  // 1080p (16:9)
		availScreenRes.push_back( Coord(1920, 1200) );  // WUXGA (16:10)
	} else {
	    // step backward through resolutions

		for(int i=0; pResolutions[i] != NULL;  i++) {
		    if(pResolutions[i]->w >= SCREEN_MIN_WIDTH && pResolutions[i]->h >= SCREEN_MIN_HEIGHT) {
		        Coord newRes = Coord(pResolutions[i]->w,pResolutions[i]->h);
		        if(std::find(availScreenRes.begin(), availScreenRes.end(), newRes) == availScreenRes.end()) {
		            // not yet in the list
		            availScreenRes.push_front(newRes);
		        }
		    }
		}
	}
}
